@@ -1,13 +1,13 @@
 /***************************************************************************
  SocNetV: Social Network Visualizer 
- version: 2.1
+ version: 2.2
  Written in Qt
 
                         main.cpp  -  description
                              -------------------
-    begin                : 9 21:10:04 EET 2005
-    copyright            : (C) 2005-2016 by Dimitris B. Kalamaras
-    email                : dimitris.kalamaras@gmail.com
+    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    project site      : http://socnetv.org
+
  ***************************************************************************/
 
 /*******************************************************************************
