@@ -1,12 +1,13 @@
 /***************************************************************************
  SocNetV: Social Network Visualizer
- version: 2.1
+ version: 2.2
  Written in Qt
  
                          node.h  -  description
                           -------------------
-    copyright            : (C) 2005-2016 by Dimitris B. Kalamaras
-    email                : dimitris.kalamaras@gmail.com
+    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    project site      : http://socnetv.org
+
  ***************************************************************************/
 
 /*******************************************************************************
@@ -97,6 +98,7 @@ public:
     void setLabelVisibility(const bool &toggle);
     void setLabelSize(const int &size);
     void setLabelText ( QString label) ;
+    void setLabelColor (const QString &color) ;
     QString labelText();
     void setLabelDistance(const int &distance);
 
