@@ -1,12 +1,13 @@
 /****************************************************************************
 SocNetV: Social Network Visualizer 
-version: 2.1
+version: 2.2
 Written in Qt
 
                                texteditor.cpp 
                              -------------------
-    copyright            : (C) 2005-2016 by Dimitris B. Kalamaras
-    email                : dimitris.kalamaras@gmail.com
+    copyright         : (C) 2005-2017 by Dimitris B. Kalamaras
+    project site      : http://socnetv.org
+
 *****************************************************************************/
 
 /*******************************************************************************
@@ -28,7 +29,9 @@ Written in Qt
 #include <QtWidgets>
 #include "texteditor.h"
 
-TextEditor::TextEditor(const QString &fileName, QWidget *parent) : QMainWindow(parent)
+TextEditor::TextEditor(const QString &fileName, QWidget *parent, const bool &format) :
+    QMainWindow(parent),
+    formatHTML(format)
 {
 	qDebug("TextEditor()");
 	textEdit = new QTextEdit;
@@ -41,9 +44,13 @@ TextEditor::TextEditor(const QString &fileName, QWidget *parent) : QMainWindow(p
 
 	readSettings();
 
-	connect(textEdit->document(), SIGNAL(contentsChanged()), this, SLOT(documentWasModified()));
+    connect(textEdit->document(), SIGNAL(contentsChanged()),
+            this, SLOT(documentWasModified()));
+
+    resize( 1024,768 );
+
+    setWindowState(Qt::WindowMaximized|Qt::WindowActive);
 
-	resize( 640,400 );
 	if (!fileName.isEmpty())
 		loadFile(fileName);
 	else 
@@ -204,7 +211,7 @@ void TextEditor::createStatusBar()
 
 void TextEditor::readSettings()
 {
-    QSettings settings("Trolltech", "Application Example");
+    QSettings settings("SocNetV", "TextEditor");
     QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
     QSize size = settings.value("size", QSize(400, 400)).toSize();
     resize(size);
@@ -213,7 +220,7 @@ void TextEditor::readSettings()
 
 void TextEditor::writeSettings()
 {
-    QSettings settings("Trolltech", "Application Example");
+    QSettings settings("SocNetV ", "TextEditor");
     settings.setValue("pos", pos());
     settings.setValue("size", size());
 }
@@ -247,7 +254,10 @@ void TextEditor::loadFile(const QString &fileName)
 
     QTextStream in(&file);
     QApplication::setOverrideCursor(Qt::WaitCursor);
-    textEdit->setPlainText(in.readAll());
+    if (formatHTML)
+     textEdit->setHtml(in.readAll());
+    else
+     textEdit->setPlainText(in.readAll());
     QApplication::restoreOverrideCursor();
 
     setCurrentFile(fileName);
@@ -269,7 +279,11 @@ bool TextEditor::saveFile(const QString &fileName)
     QTextStream outText(&file);
     outText.setCodec("UTF-8");
     QApplication::setOverrideCursor(Qt::WaitCursor);
-    outText << textEdit->toPlainText();
+    if (formatHTML)
+     outText << textEdit->toHtml();
+    else
+     outText << textEdit->toPlainText();
+
     QApplication::restoreOverrideCursor();
 
     setCurrentFile(fileName);
@@ -298,6 +312,25 @@ QString TextEditor::strippedName(const QString &fullFileName)
     return QFileInfo(fullFileName).fileName();
 }
 
+//bool TextEditor::canInsertFromMimeData( const QMimeData *source ) const
+// {
+//     if (source->hasImage())
+//         return true;
+//     else
+//         return QTextEdit::canInsertFromMimeData(source);
+// }
+
+//void TextEditor::insertFromMimeData( const QMimeData *source )
+//{
+//    if (source->hasImage())
+//    {
+//        QImage image = qvariant_cast<QImage>(source->imageData());
+//        QTextCursor cursor = this->textCursor();
+//        QTextDocument *document = this->document();
+//        document->addResource(QTextDocument::ImageResource, QUrl("image"), image);
+//        cursor.insertImage("image");
+//    }
+//}
 void TextEditor::about()
 {
    QMessageBox::about( this, "SocNetV Editor",
